# Using certificates from other apps

Certificates are installed into `/certcache/certs/` in the CertCache client container. Map a Docker volume like so:

```yaml
version: '3.7'
services:
  certcache:
    container_name: certcache
    image: ghcr.io/93million/certcache
    …
    volumes:
      - ./certcache/certs/:/certcache/certs/:rw
      …
```

Certs are stored in the format used by Certbot (regardless of whether they are generated by Certbot or returned by the thirdparty extension).

```
$ find certs
certs
certs/93m.co
certs/93m.co/chain.pem
certs/93m.co/cert.pem
certs/93m.co/privkey.pem
certs/93m.co/fullchain.pem
```

Simply map the directory through to other containers that need access to the certificates. Eg:

```yaml
  nginx:
    container_name: nginx
    image: nginx
    volumes:
      - ./certcache/certs/:/etc/certcache/certs/:ro
      - ./nginx/config/:/etc/nginx/conf.d/:rw
      -
```

You can then reference the certificates from your container. Eg:

```
server {
  listen 443 ssl http2 default_server;
  listen [::]:443 ssl http2 default_server;
  ssl_certificate /etc/certcache/certs/93m.co/fullchain.pem;
  ssl_certificate_key /etc/certcache/certs/93m.co/privkey.pem;

  server_name 93m.co;

  root /var/www;
}
```
